@@ -45,11 +45,11 @@ public:
 
     ArrayInfo(const std::string& name, void* datap, int index, const std::vector<IData>& indices,
               const std::vector<size_t>& idxWidths)
-        : m_name(name)
-        , m_datap(datap)
-        , m_index(index)
-        , m_indices(indices)
-        , m_idxWidths(idxWidths) {}
+        : m_name{name}
+        , m_datap{datap}
+        , m_index{index}
+        , m_indices{indices}
+        , m_idxWidths{idxWidths} {}
 };
 using ArrayInfoMap = std::map<std::string, std::shared_ptr<const ArrayInfo>>;
 
@@ -143,7 +143,7 @@ public:
     }
     void emitGetValue(std::ostream& s) const override {
         const int elementCounts = countMatchingElements(*m_arrVarsRefp, name());
-        for (int i = 0; i < elementCounts; i++) {
+        for (int i = 0; i < elementCounts; ++i) {
             const std::string indexed_name = name() + std::to_string(i);
             const auto it = m_arrVarsRefp->find(indexed_name);
             if (it != m_arrVarsRefp->end()) {
@@ -200,7 +200,7 @@ class VlRandomizer final {
     std::map<std::string, std::shared_ptr<const VlRandomVar>> m_vars;  // Solver-dependent
                                                                        // variables
     ArrayInfoMap m_arr_vars;  // Tracks each element in array structures for iteration
-    const VlQueue<CData>* m_randmode;  // rand_mode state;
+    const VlQueue<CData>* m_randmodep = nullptr;  // rand_mode state;
     int m_index = 0;  // Internal counter for key generation
 
     // PRIVATE METHODS
@@ -382,8 +382,9 @@ public:
 
     // Register associative array of non-struct types
     template <typename T_Key, typename T_Value>
-    void write_var(VlAssocArray<T_Key, T_Value>& var, int width, const char* name, int dimension,
-                   std::uint32_t randmodeIdx = std::numeric_limits<std::uint32_t>::max()) {
+    typename std::enable_if<!VlContainsCustomStruct<T_Value>::value, void>::type
+    write_var(VlAssocArray<T_Key, T_Value>& var, int width, const char* name, int dimension,
+              std::uint32_t randmodeIdx = std::numeric_limits<std::uint32_t>::max()) {
         if (m_vars.find(name) != m_vars.end()) return;
         m_vars[name]
             = std::make_shared<const VlRandomArrayVarTemplate<VlAssocArray<T_Key, T_Value>>>(
@@ -394,16 +395,21 @@ public:
         }
     }
 
-    // TODO: Register associative array of structs
-
+    // Register associative array of structs
+    template <typename T_Key, typename T_Value>
+    typename std::enable_if<VlContainsCustomStruct<T_Value>::value, void>::type
+    write_var(VlAssocArray<T_Key, T_Value>& var, int width, const char* name, int dimension,
+              std::uint32_t randmodeIdx = std::numeric_limits<std::uint32_t>::max()) {
+        if (dimension > 0) record_struct_arr(var, name, dimension, {}, {});
+    }
     // ----------------------------------------
     // ---  Record Arrays: flat and struct  ---
     // ----------------------------------------
 
     // Record a flat (non-class) element into the array variable table
     template <typename T>
-    typename std::enable_if<!std::is_class<T>::value, void>::type
-    record_arr_table(T& var, const std::string name, int dimension, std::vector<IData> indices,
+    typename std::enable_if<!std::is_class<T>::value || VlIsVlWide<T>::value, void>::type
+    record_arr_table(T& var, const std::string& name, int dimension, std::vector<IData> indices,
                      std::vector<size_t> idxWidths) {
         const std::string key = generateKey(name, m_index);
         m_arr_vars[key] = std::make_shared<ArrayInfo>(name, &var, m_index, indices, idxWidths);
@@ -412,7 +418,7 @@ public:
 
     // Recursively record all elements in an unpacked array
     template <typename T, std::size_t N_Depth>
-    void record_arr_table(VlUnpacked<T, N_Depth>& var, const std::string name, int dimension,
+    void record_arr_table(VlUnpacked<T, N_Depth>& var, const std::string& name, int dimension,
                           std::vector<IData> indices, std::vector<size_t> idxWidths) {
         if ((dimension > 0) && (N_Depth != 0)) {
             idxWidths.push_back(32);
@@ -428,7 +434,7 @@ public:
 
     // Recursively record all elements in a queue
     template <typename T>
-    void record_arr_table(VlQueue<T>& var, const std::string name, int dimension,
+    void record_arr_table(VlQueue<T>& var, const std::string& name, int dimension,
                           std::vector<IData> indices, std::vector<size_t> idxWidths) {
         if ((dimension > 0) && (var.size() != 0)) {
             idxWidths.push_back(32);
@@ -443,8 +449,9 @@ public:
 
     // Recursively record all elements in an associative array
     template <typename T_Key, typename T_Value>
-    void record_arr_table(VlAssocArray<T_Key, T_Value>& var, const std::string name, int dimension,
-                          std::vector<IData> indices, std::vector<size_t> idxWidths) {
+    void record_arr_table(VlAssocArray<T_Key, T_Value>& var, const std::string& name,
+                          int dimension, std::vector<IData> indices,
+                          std::vector<size_t> idxWidths) {
         if ((dimension > 0) && (var.size() != 0)) {
             for (auto it = var.begin(); it != var.end(); ++it) {
                 const T_Key& key = it->first;
@@ -472,22 +479,25 @@ public:
     // Register a single structArray element via write_var
     template <typename T>
     typename std::enable_if<VlContainsCustomStruct<T>::value, void>::type
-    record_struct_arr(T& var, const std::string name, int dimension, std::vector<IData> indices,
+    record_struct_arr(T& var, const std::string& name, int dimension, std::vector<IData> indices,
                       std::vector<size_t> idxWidths) {
         std::ostringstream oss;
         for (size_t i = 0; i < indices.size(); ++i) {
-            oss << std::hex << static_cast<int>(indices[i]);
+            oss << std::hex << std::setw(int(idxWidths[i] / 4)) << std::setfill('0')
+                << static_cast<int>(indices[i]);
             if (i < indices.size() - 1) oss << ".";
         }
-        write_var(var, 1ULL, (name + "." + oss.str()).c_str(), 1ULL);
+        write_var(var, 1ULL,
+                  oss.str().length() > 0 ? (name + "." + oss.str()).c_str() : name.c_str(), 1ULL);
     }
 
     // Recursively process VlUnpacked of structs
     template <typename T, std::size_t N_Depth>
-    void record_struct_arr(VlUnpacked<T, N_Depth>& var, const std::string name, int dimension,
+    void record_struct_arr(VlUnpacked<T, N_Depth>& var, const std::string& name, int dimension,
                            std::vector<IData> indices, std::vector<size_t> idxWidths) {
         if (dimension > 0 && N_Depth != 0) {
-            idxWidths.push_back(32);
+            constexpr size_t idx_width = 1 << VL_CLOG2_CE_Q(VL_CLOG2_CE_Q(N_Depth) + 1);
+            idxWidths.push_back(idx_width);
             for (size_t i = 0; i < N_Depth; ++i) {
                 indices.push_back(i);
                 record_struct_arr(var.operator[](i), name, dimension - 1, indices, idxWidths);
@@ -498,7 +508,7 @@ public:
 
     // Recursively process VlQueue of structs
     template <typename T>
-    void record_struct_arr(VlQueue<T>& var, const std::string name, int dimension,
+    void record_struct_arr(VlQueue<T>& var, const std::string& name, int dimension,
                            std::vector<IData> indices, std::vector<size_t> idxWidths) {
         if ((dimension > 0) && (var.size() != 0)) {
             idxWidths.push_back(32);
@@ -510,16 +520,39 @@ public:
         }
     }
 
-    // TODO: Add support for associative arrays of structs
     // Recursively process associative arrays of structs
+    template <typename T_Key, typename T_Value>
+    void record_struct_arr(VlAssocArray<T_Key, T_Value>& var, const std::string& name,
+                           int dimension, const std::vector<IData>& indices,
+                           const std::vector<size_t>& idxWidths) {
+        if ((dimension > 0) && (!var.empty())) {
+            for (auto it = var.begin(); it != var.end(); ++it) {
+                const T_Key& key = it->first;
+                const T_Value& value = it->second;
 
+                std::string indexed_name;
+                std::vector<size_t> integral_index;
+                size_t idx_width = 0;
+
+                process_key(key, indexed_name, integral_index, name, idx_width);
+                std::ostringstream oss;
+                for (int i = 0; i < integral_index.size(); ++i)
+                    oss << std::hex << static_cast<int>(integral_index[i]);
+
+                std::string result = oss.str();
+                result.insert(result.begin(), int(idx_width / 4) - result.size(), '0');
+                record_struct_arr(var.at(key), name + "." + result, dimension - 1, indices,
+                                  idxWidths);
+            }
+        }
+    }
     // --------------------------
     // ---  Helper functions  ---
     // --------------------------
 
     // Helper: Register all members of a user-defined struct
     template <typename T, std::size_t... I>
-    void modifyMembers(T& obj, std::index_sequence<I...>, std::string baseName) {
+    void modifyMembers(T& obj, std::index_sequence<I...>, const std::string& baseName) {
         // Use the indices to access each member via std::get
         (void)std::initializer_list<int>{
             (write_var(std::get<I>(obj.getMembers(obj)), obj.memberWidth()[I],
@@ -541,10 +574,28 @@ public:
 
     void hard(std::string&& constraint);
     void clear();
-    void set_randmode(const VlQueue<CData>& randmode) { m_randmode = &randmode; }
+    void set_randmode(const VlQueue<CData>& randmode) { m_randmodep = &randmode; }
 #ifdef VL_DEBUG
     void dump() const;
 #endif
 };
 
+//=============================================================================
+// VlStdRandomizer provides a light wrapper for RNG used by std::randomize()
+// to support scope-level randomization.
+class VlStdRandomizer final {
+    // MEMBERS
+    VlRNG m_rng;  // Random number generator
+
+public:
+    // CONSTRUCTORS
+    VlStdRandomizer() = default;
+    ~VlStdRandomizer() = default;
+
+    template <typename T>
+    bool basicStdRandomization(T& value, size_t width) {
+        value = VL_MASK_I(width) & VL_RANDOM_RNG_I(m_rng);
+        return true;
+    }
+};
 #endif  // Guard
